/*

*/
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <Ticker.h>
#include <Adafruit_PWMServoDriver.h>

//常量
#define VER               "Doit_Servo_V1.0"
#define DEFAULT_AP_SSID   "Doit_Servo"
#define DEFAULT_AP_PSW    "12345678"
#define PINLED            16
#define PINKEY            0
#define UDPPORT           6090

#define DebugSerial   Serial
#define MAX_PACKETSIZE 512

//nodemcu dev kit
// #define PinPWMA 5
// #define PinPWMB 4
// #define PinDirA 0
// #define PinDirB 2
#define PinPWMA 14
#define PinPWMB 12
#define PinDirA 15
#define PinDirB 13


Ticker timer;
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

int spdTargetA = 1023;
int spdTargetB = 1023;
bool stopFlag = true;
bool JogFlag = false;
uint16_t JogTimeCnt = 0;


IPAddress apIP(192, 168, 4, 1);
IPAddress netMsk(255, 255, 255, 0);

void parseUDPPackage(char *p);

/***********************************************
  *UPD通讯
  端口：6090
  IP地址：255.255.255.255
***********************************************/
WiFiUDP udp;
char buffUDP[MAX_PACKETSIZE];

/*
  *开始UDP服务
 */
void startUDPServer(int port)
{
  DebugSerial.print("\r\nStartUDPServer at port:");
  DebugSerial.println(port);
  udp.begin(port);
}

/*
  *监听读取udp包
 */
void doUdpServerTick()
{
  if (udp.parsePacket())
  {
    memset(buffUDP, 0x00, sizeof(buffUDP));
    udp.read(buffUDP, MAX_PACKETSIZE - 1);
    udp.flush();
    parseUDPPackage(buffUDP);
  }
}

/*
  *发送udp包
 */
void sendUDP(char *p)
{
  udp.beginPacket(udp.remoteIP(), udp.remotePort());
  udp.write(p); udp.endPacket();
}

/*
  电机速度控制
*/
void tick_car_control()
{
  if (JogFlag == true) {
    stopFlag = false;
    if (JogTimeCnt <= 0) {
      JogFlag = false; stopFlag = true;
    }
    JogTimeCnt--;
  }
  if (stopFlag == false) {
    analogWrite(PinPWMA, spdTargetA);
    analogWrite(PinPWMB, spdTargetA);
  }
  else {
    JogTimeCnt = 0;
    analogWrite(PinPWMA, 0);
    analogWrite(PinPWMB, 0);
  }
}

/*
  初始化电机参数
*/
void initParseData()
{
  Serial.println("[initParseData]");
  pinMode(PinPWMA, OUTPUT); pinMode(PinPWMB, OUTPUT);
  pinMode(PinDirA, OUTPUT); pinMode(PinDirB, OUTPUT);
  digitalWrite(PinDirA, HIGH); digitalWrite(PinDirB, HIGH);
  digitalWrite(PinPWMA, LOW); digitalWrite(PinPWMB, LOW);
  timer.attach(0.1, tick_car_control);
  
  pwm.begin();
  pwm.setPWMFreq(60);
}

/*
  UDP接收包处理
*/
void parseUDPPackage(char *p)
{
  Serial.print("[UDP parseData:]");
  Serial.println(p);
  if (strstr(p, "cmd=motor&d=") != NULL) {
    char *s = p + strlen("cmd=motor&d="); if (*s == 0x00) return;
    switch (*s) {
      case '0': analogWrite(PinPWMA, 0); analogWrite(PinPWMB, 0); stopFlag = true; JogTimeCnt = 0; break;
      case '1': digitalWrite(PinDirA, HIGH); digitalWrite(PinDirB, HIGH); JogTimeCnt = 50; JogFlag = true; break;
      case '2': digitalWrite(PinDirA, LOW); digitalWrite(PinDirB, LOW); JogTimeCnt = 50; JogFlag = true; break;
      case '3': digitalWrite(PinDirA, LOW); digitalWrite(PinDirB, HIGH); JogTimeCnt = 50; JogFlag = true; break;
      case '4': digitalWrite(PinDirA, HIGH); digitalWrite(PinDirB, LOW); JogTimeCnt = 50; JogFlag = true; break;
      case '5': spdTargetA += 100; if (spdTargetA > 1023) spdTargetA = 1023; spdTargetB = spdTargetA; break;
      case '6': spdTargetA -= 100; if (spdTargetA < 0) spdTargetA = 0; spdTargetB = spdTargetA; break;
      case 'A': char t[32]; sprintf(t, "cmd=motor&spd=%d", spdTargetA); sendUDP(t); break;
      default: break;
    }
    if (*s != 5) sendUDP((char*)"CMD OK");
  }
  else if(strstr(p, "cmd=servo") != NULL) {
    char *s=NULL,*e=NULL;
    s = strstr(p, "&n=");
    if (s == NULL) return;
    e = strstr(s + 3, "&d=");
    if (e == NULL) return;
    char servonum[10]; memset(servonum, 0x00, 5);
    memcpy(servonum, s + 3, e - s - 3);
    char pulselen[10]; memset(pulselen, 0x00, 5);
    strcpy(pulselen, e + 3);
    
    Serial.printf("servonum=%s,pulselen=%s\r\n",servonum,pulselen);
    pwm.setPWM(atoi(servonum), 0, atoi(pulselen));
    sendUDP((char*)"CMD OK");
  }
}

/*
  初始化WiFi
*/
void initWiFi()
{
  char temp[32];
  uint8_t mac[6];
  WiFi.macAddress(mac);
  sprintf(temp, "%s_%02X%02X%02X", DEFAULT_AP_SSID, mac[3], mac[4], mac[5]);
  WiFi.mode(WIFI_AP);
  WiFi.softAP(temp,DEFAULT_AP_PSW);
  WiFi.softAPConfig(apIP, apIP, netMsk);
  delay(100);
}

/*
  初始化
*/
void setup() {
  DebugSerial.begin (115200);
  
  pinMode(PINLED, OUTPUT);
  DebugSerial.println("\r\n\r\nStart ESP8266");
  DebugSerial.printf("Firmware Version:%s\r\n",VER);
  DebugSerial.printf("SDK Version:%s\r\n",ESP.getSdkVersion());
  ESP.wdtEnable(5000);

  initWiFi();
  startUDPServer(UDPPORT);
  initParseData();
}

/*
  循环
*/
void loop() {
  ESP.wdtFeed();
  doUdpServerTick();
}